package be.c4j.graphqldemo.graphqldemo.graphql;

import graphql.GraphQL;
import graphql.schema.GraphQLSchema;
import graphql.schema.idl.*;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.IOException;

import static graphql.schema.idl.RuntimeWiring.newRuntimeWiring;

@Component
public class GraphQLService {

    @Value("${be.c4j.graphqldemo.schemaName}")
    private String schemaName;

    private GraphQLDataFetchers graphQLDataFetchers;
    private GraphQL graphQL;

    public GraphQLService(GraphQLDataFetchers graphQLDataFetchers) {
        this.graphQLDataFetchers = graphQLDataFetchers;
    }

    @PostConstruct
    public void init() throws IOException {
        final ClassPathResource classPathResource = new ClassPathResource(schemaName);
        TypeDefinitionRegistry typeRegistry = new SchemaParser().parse(classPathResource.getFile());


        RuntimeWiring runtimeWiring = newRuntimeWiring()
                .type(TypeRuntimeWiring.newTypeWiring("Query").dataFetcher("product", graphQLDataFetchers.productDataFetcher))
                .type(TypeRuntimeWiring.newTypeWiring("Query").dataFetcher("order", graphQLDataFetchers.orderDataFetcher))
                .build();

        SchemaGenerator schemaGenerator = new SchemaGenerator();
        GraphQLSchema graphQLSchema  = schemaGenerator.makeExecutableSchema(typeRegistry, runtimeWiring);


        graphQL = GraphQL.newGraphQL(graphQLSchema).build();
    }

    @Bean
    public GraphQL graphQL() {
        return graphQL;
    }

}
